/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasGradient.h"
#include "CanvasPixelArray.h"
#include "CanvasRenderingContext2D.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "ImageData.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "TextMetrics.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCanvasGradient.h"
#include "gdom/GdomCanvasRenderingContext2D.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomImageData.h"
#include "gdom/GdomTextMetrics.h"
#include "gdom/PrivateGdomCanvasGradient.h"
#include "gdom/PrivateGdomCanvasRenderingContext2D.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomImageData.h"
#include "gdom/PrivateGdomTextMetrics.h"
#include "gdom/PrivateIntGdomCanvasGradient.h"
#include "gdom/PrivateIntGdomCanvasRenderingContext2D.h"
#include "gdom/PrivateIntGdomImageData.h"
#include "gdom/PrivateIntGdomTextMetrics.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::CanvasRenderingContext2D* obj)
{
    return WebCore::cacheDOMObject<WebCore::CanvasRenderingContext2D, GDOMCanvasRenderingContext2D>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMCanvasRenderingContext2D::GDOMCanvasRenderingContext2D(PassRefPtr<WebCore::CanvasRenderingContext2D> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMCanvasRenderingContext2D::~GDOMCanvasRenderingContext2D()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

void
gdom_canvas_rendering_context2d_save (GdomCanvasRenderingContext2D *thiz)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->save();

}

void
gdom_canvas_rendering_context2d_restore (GdomCanvasRenderingContext2D *thiz)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->restore();

}

void
gdom_canvas_rendering_context2d_scale (GdomCanvasRenderingContext2D *thiz, gfloat sx, gfloat sy)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->scale(sx, sy);

}

void
gdom_canvas_rendering_context2d_rotate (GdomCanvasRenderingContext2D *thiz, gfloat angle)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->rotate(angle);

}

void
gdom_canvas_rendering_context2d_translate (GdomCanvasRenderingContext2D *thiz, gfloat tx, gfloat ty)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->translate(tx, ty);

}

void
gdom_canvas_rendering_context2d_transform (GdomCanvasRenderingContext2D *thiz, gfloat m11, gfloat m12, gfloat m21, gfloat m22, gfloat dx, gfloat dy)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->transform(m11, m12, m21, m22, dx, dy);

}

GdomCanvasGradient *
gdom_canvas_rendering_context2d_create_linear_gradient (GdomCanvasRenderingContext2D *thiz, gfloat x0, gfloat y0, gfloat x1, gfloat y1)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::CanvasGradient> g_res = WTF::getPtr(item->createLinearGradient(x0, y0, x1, y1));
    GdomCanvasGradient * res = static_cast<GdomCanvasGradient *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomCanvasGradient *
gdom_canvas_rendering_context2d_create_radial_gradient (GdomCanvasRenderingContext2D *thiz, gfloat x0, gfloat y0, gfloat r0, gfloat x1, gfloat y1, gfloat r1)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::CanvasGradient> g_res = WTF::getPtr(item->createRadialGradient(x0, y0, r0, x1, y1, r1));
    GdomCanvasGradient * res = static_cast<GdomCanvasGradient *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_canvas_rendering_context2d_clear_rect (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y, gfloat width, gfloat height)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->clearRect(x, y, width, height);

}

void
gdom_canvas_rendering_context2d_fill_rect (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y, gfloat width, gfloat height)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->fillRect(x, y, width, height);

}

void
gdom_canvas_rendering_context2d_begin_path (GdomCanvasRenderingContext2D *thiz)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->beginPath();

}

void
gdom_canvas_rendering_context2d_close_path (GdomCanvasRenderingContext2D *thiz)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->closePath();

}

void
gdom_canvas_rendering_context2d_move_to (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->moveTo(x, y);

}

void
gdom_canvas_rendering_context2d_line_to (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->lineTo(x, y);

}

void
gdom_canvas_rendering_context2d_quadratic_curve_to (GdomCanvasRenderingContext2D *thiz, gfloat cpx, gfloat cpy, gfloat x, gfloat y)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->quadraticCurveTo(cpx, cpy, x, y);

}

void
gdom_canvas_rendering_context2d_bezier_curve_to (GdomCanvasRenderingContext2D *thiz, gfloat cp1x, gfloat cp1y, gfloat cp2x, gfloat cp2y, gfloat x, gfloat y)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->bezierCurveTo(cp1x, cp1y, cp2x, cp2y, x, y);

}

void
gdom_canvas_rendering_context2d_arc_to (GdomCanvasRenderingContext2D *thiz, gfloat x1, gfloat y1, gfloat x2, gfloat y2, gfloat radius)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->arcTo(x1, y1, x2, y2, radius, ec);

}

void
gdom_canvas_rendering_context2d_rect (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y, gfloat width, gfloat height)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->rect(x, y, width, height);

}

void
gdom_canvas_rendering_context2d_arc (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y, gfloat radius, gfloat start_angle, gfloat end_angle, gboolean anticlockwise)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    WebCore::ExceptionCode ec = 0;
    item->arc(x, y, radius, start_angle, end_angle, anticlockwise, ec);

}

void
gdom_canvas_rendering_context2d_fill (GdomCanvasRenderingContext2D *thiz)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->fill();

}

void
gdom_canvas_rendering_context2d_stroke (GdomCanvasRenderingContext2D *thiz)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->stroke();

}

void
gdom_canvas_rendering_context2d_clip (GdomCanvasRenderingContext2D *thiz)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->clip();

}

gboolean
gdom_canvas_rendering_context2d_is_point_in_path (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_val_if_fail (item, 0);
    gboolean res = item->isPointInPath(x, y);
    return res;

}


/* XXX TODO: custom function gdom_canvas_rendering_context2d_fill_text */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_stroke_text */

GdomTextMetrics *
gdom_canvas_rendering_context2d_measure_text (GdomCanvasRenderingContext2D *thiz, gchar * text)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (text, 0);
    WebCore::String _g_text = WebCore::String::fromUTF8(text);
    PassRefPtr<WebCore::TextMetrics> g_res = WTF::getPtr(item->measureText(_g_text));
    GdomTextMetrics * res = static_cast<GdomTextMetrics *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_canvas_rendering_context2d_set_alpha (GdomCanvasRenderingContext2D *thiz, gfloat alpha)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->setAlpha(alpha);

}

void
gdom_canvas_rendering_context2d_set_composite_operation (GdomCanvasRenderingContext2D *thiz, gchar * composite_operation)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (composite_operation);
    WebCore::String _g_composite_operation = WebCore::String::fromUTF8(composite_operation);
    item->setCompositeOperation(_g_composite_operation);

}

void
gdom_canvas_rendering_context2d_set_line_width (GdomCanvasRenderingContext2D *thiz, gfloat width)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->setLineWidth(width);

}

void
gdom_canvas_rendering_context2d_set_line_cap (GdomCanvasRenderingContext2D *thiz, gchar * cap)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (cap);
    WebCore::String _g_cap = WebCore::String::fromUTF8(cap);
    item->setLineCap(_g_cap);

}

void
gdom_canvas_rendering_context2d_set_line_join (GdomCanvasRenderingContext2D *thiz, gchar * join)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (join);
    WebCore::String _g_join = WebCore::String::fromUTF8(join);
    item->setLineJoin(_g_join);

}

void
gdom_canvas_rendering_context2d_set_miter_limit (GdomCanvasRenderingContext2D *thiz, gfloat limit)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->setMiterLimit(limit);

}

void
gdom_canvas_rendering_context2d_clear_shadow (GdomCanvasRenderingContext2D *thiz)
{
    g_return_if_fail (thiz);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_if_fail (item);
    item->clearShadow();

}


/* XXX TODO: custom function gdom_canvas_rendering_context2d_set_stroke_color */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_set_fill_color */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_stroke_rect */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_draw_image */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_draw_image_from_rect */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_set_shadow */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_create_pattern */

GdomImageData *
gdom_canvas_rendering_context2d_create_image_data (GdomCanvasRenderingContext2D *thiz, gfloat sw, gfloat sh)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::ImageData> g_res = WTF::getPtr(item->createImageData(sw, sh));
    GdomImageData * res = static_cast<GdomImageData *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomImageData *
gdom_canvas_rendering_context2d_get_image_data (GdomCanvasRenderingContext2D *thiz, gfloat sx, gfloat sy, gfloat sw, gfloat sh)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::CanvasRenderingContext2D * item = (gdom_canvas_rendering_context2d_get_private(thiz));
    g_return_val_if_fail (item, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::ImageData> g_res = WTF::getPtr(item->getImageData(sx, sy, sw, sh, ec));
    GdomImageData * res = static_cast<GdomImageData *>(WebCore::toGDOM(g_res.get()));
    return res;

}


/* XXX TODO: custom function gdom_canvas_rendering_context2d_put_image_data */

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "CanvasRenderingContext2D.h"
#include <gdom/GdomCanvasRenderingContext2D.h>
#include <gdom/PrivateGdomCanvasRenderingContext2D.h>
#include <gdom/PrivateIntGdomCanvasRenderingContext2D.h>
#include <gdom/GdomCanvasRenderingContext2Dprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomCanvasRenderingContext2D, gdom_canvas_rendering_context2d, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomCanvasRenderingContext2DPrivate {
    class WebCore::CanvasRenderingContext2D *item;
};

#define GDOM_CANVAS_RENDERING_CONTEXT2D_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_CANVAS_RENDERING_CONTEXT2D, GdomCanvasRenderingContext2DPrivate))
*/
G_END_DECLS
WebCore::CanvasRenderingContext2D *
gdom_canvas_rendering_context2d_get_private(GdomCanvasRenderingContext2D *request)
{
    GDOMCanvasRenderingContext2D *res;
    res = static_cast<GDOMCanvasRenderingContext2D*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::CanvasRenderingContext2D*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_CANVAS,
    PROP_GLOBAL_ALPHA,
    PROP_GLOBAL_COMPOSITE_OPERATION,
    PROP_LINE_WIDTH,
    PROP_LINE_CAP,
    PROP_LINE_JOIN,
    PROP_MITER_LIMIT,
    PROP_SHADOW_OFFSET_X,
    PROP_SHADOW_OFFSET_Y,
    PROP_SHADOW_BLUR,
    PROP_SHADOW_COLOR,
    PROP_FONT,
    PROP_TEXT_ALIGN,
    PROP_TEXT_BASELINE,
};


static void gdom_canvas_rendering_context2d_finalize(GObject* object)
{
    /*GdomCanvasRenderingContext2D* request = GDOM_CANVAS_RENDERING_CONTEXT2D(object);*/
    /*GdomCanvasRenderingContext2DPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_canvas_rendering_context2d_parent_class)->finalize(object);
}

static void gdom_canvas_rendering_context2d__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomCanvasRenderingContext2D *self = GDOM_CANVAS_RENDERING_CONTEXT2D(object);
    WebCore::CanvasRenderingContext2D * gself = (gdom_canvas_rendering_context2d_get_private(self));
    switch (prop_id) {
     case PROP_GLOBAL_ALPHA:
    {
         gself->setGlobalAlpha((g_value_get_float (value)) );
         break;
    }
     case PROP_GLOBAL_COMPOSITE_OPERATION:
    {
         gself->setGlobalCompositeOperation(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_LINE_WIDTH:
    {
         gself->setLineWidth((g_value_get_float (value)) );
         break;
    }
     case PROP_LINE_CAP:
    {
         gself->setLineCap(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_LINE_JOIN:
    {
         gself->setLineJoin(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_MITER_LIMIT:
    {
         gself->setMiterLimit((g_value_get_float (value)) );
         break;
    }
     case PROP_SHADOW_OFFSET_X:
    {
         gself->setShadowOffsetX((g_value_get_float (value)) );
         break;
    }
     case PROP_SHADOW_OFFSET_Y:
    {
         gself->setShadowOffsetY((g_value_get_float (value)) );
         break;
    }
     case PROP_SHADOW_BLUR:
    {
         gself->setShadowBlur((g_value_get_float (value)) );
         break;
    }
     case PROP_SHADOW_COLOR:
    {
         gself->setShadowColor(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_FONT:
    {
         gself->setFont(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_TEXT_ALIGN:
    {
         gself->setTextAlign(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_TEXT_BASELINE:
    {
         gself->setTextBaseline(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_canvas_rendering_context2d__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomCanvasRenderingContext2D *self = GDOM_CANVAS_RENDERING_CONTEXT2D(object);
    WebCore::CanvasRenderingContext2D * gself = (gdom_canvas_rendering_context2d_get_private(self));
    switch (prop_id) {
    case PROP_CANVAS:
    {
         PassRefPtr<WebCore::HTMLCanvasElement> ptr = gself->canvas();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_GLOBAL_ALPHA:
    {
         g_value_set_float (value, gself->globalAlpha());
         break;
    }
    case PROP_GLOBAL_COMPOSITE_OPERATION:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->globalCompositeOperation()));
         break;
    }
    case PROP_LINE_WIDTH:
    {
         g_value_set_float (value, gself->lineWidth());
         break;
    }
    case PROP_LINE_CAP:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->lineCap()));
         break;
    }
    case PROP_LINE_JOIN:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->lineJoin()));
         break;
    }
    case PROP_MITER_LIMIT:
    {
         g_value_set_float (value, gself->miterLimit());
         break;
    }
    case PROP_SHADOW_OFFSET_X:
    {
         g_value_set_float (value, gself->shadowOffsetX());
         break;
    }
    case PROP_SHADOW_OFFSET_Y:
    {
         g_value_set_float (value, gself->shadowOffsetY());
         break;
    }
    case PROP_SHADOW_BLUR:
    {
         g_value_set_float (value, gself->shadowBlur());
         break;
    }
    case PROP_SHADOW_COLOR:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->shadowColor()));
         break;
    }
    case PROP_FONT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->font()));
         break;
    }
    case PROP_TEXT_ALIGN:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->textAlign()));
         break;
    }
    case PROP_TEXT_BASELINE:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->textBaseline()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_canvas_rendering_context2d_class_init(GdomCanvasRenderingContext2DClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_canvas_rendering_context2d_finalize;
    g_klass->set_property = gdom_canvas_rendering_context2d__set_property_fn;
    g_klass->get_property = gdom_canvas_rendering_context2d__get_property_fn;

    g_object_class_install_property (g_klass, PROP_CANVAS,
            g_param_spec_object ("canvas", /* name */
                "canvas_rendering_context2d_canvas", /* short description */
                "read-only  GdomHTMLCanvasElement * CanvasRenderingContext2D.canvas", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HTML_CANVAS_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_GLOBAL_ALPHA,
            g_param_spec_float ("global_alpha", /* name */
                "canvas_rendering_context2d_global_alpha", /* short description */
                "read-write  gfloat CanvasRenderingContext2D.global_alpha", /* longer - could do with some extra doc stuff here */
                 G_MINFLOAT,        /* min */
                G_MAXFLOAT, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_GLOBAL_COMPOSITE_OPERATION,
            g_param_spec_string ("global_composite_operation", /* name */
                "canvas_rendering_context2d_global_composite_operation", /* short description */
                "read-write  gchar * CanvasRenderingContext2D.global_composite_operation", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LINE_WIDTH,
            g_param_spec_float ("line_width", /* name */
                "canvas_rendering_context2d_line_width", /* short description */
                "read-write  gfloat CanvasRenderingContext2D.line_width", /* longer - could do with some extra doc stuff here */
                 G_MINFLOAT,        /* min */
                G_MAXFLOAT, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LINE_CAP,
            g_param_spec_string ("line_cap", /* name */
                "canvas_rendering_context2d_line_cap", /* short description */
                "read-write  gchar * CanvasRenderingContext2D.line_cap", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LINE_JOIN,
            g_param_spec_string ("line_join", /* name */
                "canvas_rendering_context2d_line_join", /* short description */
                "read-write  gchar * CanvasRenderingContext2D.line_join", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_MITER_LIMIT,
            g_param_spec_float ("miter_limit", /* name */
                "canvas_rendering_context2d_miter_limit", /* short description */
                "read-write  gfloat CanvasRenderingContext2D.miter_limit", /* longer - could do with some extra doc stuff here */
                 G_MINFLOAT,        /* min */
                G_MAXFLOAT, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SHADOW_OFFSET_X,
            g_param_spec_float ("shadow_offset_x", /* name */
                "canvas_rendering_context2d_shadow_offset_x", /* short description */
                "read-write  gfloat CanvasRenderingContext2D.shadow_offset_x", /* longer - could do with some extra doc stuff here */
                 G_MINFLOAT,        /* min */
                G_MAXFLOAT, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SHADOW_OFFSET_Y,
            g_param_spec_float ("shadow_offset_y", /* name */
                "canvas_rendering_context2d_shadow_offset_y", /* short description */
                "read-write  gfloat CanvasRenderingContext2D.shadow_offset_y", /* longer - could do with some extra doc stuff here */
                 G_MINFLOAT,        /* min */
                G_MAXFLOAT, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SHADOW_BLUR,
            g_param_spec_float ("shadow_blur", /* name */
                "canvas_rendering_context2d_shadow_blur", /* short description */
                "read-write  gfloat CanvasRenderingContext2D.shadow_blur", /* longer - could do with some extra doc stuff here */
                 G_MINFLOAT,        /* min */
                G_MAXFLOAT, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SHADOW_COLOR,
            g_param_spec_string ("shadow_color", /* name */
                "canvas_rendering_context2d_shadow_color", /* short description */
                "read-write  gchar * CanvasRenderingContext2D.shadow_color", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_FONT,
            g_param_spec_string ("font", /* name */
                "canvas_rendering_context2d_font", /* short description */
                "read-write  gchar * CanvasRenderingContext2D.font", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_TEXT_ALIGN,
            g_param_spec_string ("text_align", /* name */
                "canvas_rendering_context2d_text_align", /* short description */
                "read-write  gchar * CanvasRenderingContext2D.text_align", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_TEXT_BASELINE,
            g_param_spec_string ("text_baseline", /* name */
                "canvas_rendering_context2d_text_baseline", /* short description */
                "read-write  gchar * CanvasRenderingContext2D.text_baseline", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomCanvasRenderingContext2DPrivate));*/
}

static void gdom_canvas_rendering_context2d_init(GdomCanvasRenderingContext2D* request)
{
    /*GdomCanvasRenderingContext2DPrivate* priv = GDOM_CANVAS_RENDERING_CONTEXT2D_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

